package com.lhsystems.iocc.test;

import java.util.Date;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

/**
 * Base class of test cases for HornetQ topic testing.
 * 
 * @author u292076
 * 
 */
public abstract class HornetQTopicTester {

	private static Logger log = Logger
			.getLogger(HornetQTopicGroupIdTester.class);
	private int count;
	InitialContext ic = null;
	ConnectionFactory cf;

	protected int getReceivedMessageCount() {
		return count;
	}

	protected static Properties createProps(String jnpUrl) {
		Properties props = new Properties();
		props.setProperty("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		props.setProperty("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		props.setProperty("java.naming.provider.url", jnpUrl);
		return props;
	}

	protected synchronized Connection connect(String jnpUrl, String connFact,
			String topicName, String clientId, String subsName)
			throws JMSException, NamingException {
		return connect(jnpUrl, connFact, topicName, clientId, subsName, false);
	}
	
	protected synchronized Connection connect(String jnpUrl, String connFact,
			String topicName, String clientId, String subsName, boolean createNewCF)
			throws JMSException, NamingException {
		Connection connection = null;

		if (ic == null) {
			ic = new InitialContext(createProps(jnpUrl));
		}
		Topic topic = (Topic) ic.lookup(topicName);
		connection = getConnection(ic, connFact, clientId, createNewCF);

		Session session0 = connection.createSession(false,
				Session.AUTO_ACKNOWLEDGE);

		MessageConsumer subscriber = session0.createDurableSubscriber(topic,
				subsName);

		subscriber.setMessageListener(getMessageListener(clientId));
		connection.setExceptionListener(getExceptionListener());

		connection.start();

		log.info("Client " + clientId + " now starts receiving.");
		return connection;

	}

	private Connection getConnection(InitialContext ic, String connFact,
			String clientId) throws NamingException, JMSException {
		return getConnection(ic, connFact, clientId, false);
	}
	private Connection getConnection(InitialContext ic, String connFact,
			String clientId, boolean alwaysCreateNewCF) throws NamingException, JMSException {
		Connection connection;
		// ConnectionFactory cf = (ConnectionFactory) ic.lookup(connFact);
		// If ConnectionFactory is reused than connections are ALWAYS created in
		// round robin fashion to one of the nodes.
		if (cf == null || alwaysCreateNewCF) {
			cf = (ConnectionFactory) ic.lookup(connFact);
		}

		connection = cf.createConnection();
		connection.setClientID(clientId);
		return connection;
	}

	protected void closeConnection(Connection connection) {
		log.info("Closing connection");
		try {
			if (connection != null) {
				connection.close();
			}
		} catch (JMSException e) {
			log.error("Closing connection unsuccesful", e);
		}

	}

	protected void unsubscribe(String jnpUrl, String connFact, String clientId,
			String subsName) throws NamingException, JMSException {
		log.info("Unsubscribing client " + clientId);

		InitialContext ic = null;
		Connection connection = null;

		try {
			ic = new InitialContext(createProps(jnpUrl));
			connection = getConnection(ic, connFact, clientId, true);

			Session session = connection.createSession(false,
					Session.AUTO_ACKNOWLEDGE);

			session.unsubscribe(subsName);
		} finally {
			if (connection != null) {
				connection.close();
			}

			if (ic != null) {
				ic.close();
			}

		}
	}

	protected ExceptionListener getExceptionListener() {
		return new JmsExceptionListener();
	}

	protected class JmsExceptionListener implements ExceptionListener {

		@Override
		public void onException(final JMSException ex) {
			// log.info("Exception, try to reconnect ... ");
			log.info("Exception, no reconnection by hand ... ");
			log.info("Exception was: ", ex);

			// scheduleReconnect();
		}
	}

	// private void scheduleReconnect() {
	// log.debug("Scheduling reconnect Task");
	// reconnectExecutor.schedule(new ReconnectTask(1), TASK_RESTART_PERIOD *
	// 20,
	// TimeUnit.MILLISECONDS);
	// }
	//
	// private class ReconnectTask implements Runnable {
	//
	// private int timeoutSec;
	//
	// public ReconnectTask(int timeoutSec) {
	// setTimeoutSec(timeoutSec);
	// }
	//
	// @Override
	// public synchronized void run() {
	// try {
	// log.info("Reconnecting ...");
	// connect();
	// return;
	// } catch (final JMSException e) {
	// log.error("got JMS exception during reconnect...", e);
	// } catch (final NamingException e) {
	// log.error("got Naming exception during reconnect...", e);
	// }
	// setTimeoutSec(timeoutSec * 2);
	//
	// log.error("New reconnect try in " + timeoutSec + " sec.");
	// reconnectExecutor.schedule(this, timeoutSec , TimeUnit.SECONDS);
	// }
	//
	// private void setTimeoutSec(int timeoutSec) {
	// this.timeoutSec = Math.max(1, Math.min(5 * 60, timeoutSec));
	// }
	// }
	//
	protected MessageListener getMessageListener(String clientId) {
		return new GroupIdMessageListener(clientId);
	}

	protected class GroupIdMessageListener implements MessageListener {
		private String clientId;

		public GroupIdMessageListener(String clientId) {
			super();
			this.clientId = clientId;
		}

		@Override
		public void onMessage(Message arg0) {

			try {
				TextMessage textMessage = (TextMessage) arg0;
				log.info("Receiver " + clientId + " got " + ++count
						+ " messages. " + textMessage.getText() + ", groupID:"
						+ textMessage.getStringProperty("JMSXGroupID"));
			} catch (JMSException e) {
				e.printStackTrace();
			}

		}

	}

	protected class TopicSender implements Runnable {
		private Logger log = Logger.getLogger(TopicSender.class);
		private final String CLIENT_ID_PUB = "TestClientPub";

		private int timeoutMillis;
		private int numMessages;
		private int reconnects = 0;
		private int messageCount = 0;

		private String jnpUrl;
		private String connFact;
		private String topicName;

		public TopicSender(int timeoutMillis, int numMessages, String jnpUrl,
				String connFact, String topicName) {
			this.timeoutMillis = timeoutMillis;
			this.numMessages = numMessages;
			this.jnpUrl = jnpUrl;
			this.connFact = connFact;
			this.topicName = topicName;
		}

		@Override
		public void run() {
			log.info("TopicSender is starting. We will send " + numMessages
					+ " messages.");
			InitialContext ic = null;
			Connection connection = null;

			try {
				ic = new InitialContext(createProps(jnpUrl));

				Topic topic = (Topic) ic.lookup(topicName);

				ConnectionFactory cf = (ConnectionFactory) ic.lookup(connFact);

				connection = cf.createConnection();

				connection.setClientID(CLIENT_ID_PUB);

				Session session0 = connection.createSession(false,
						Session.AUTO_ACKNOWLEDGE);

				connection.start();

				MessageProducer producer = session0.createProducer(topic);

				for (int i = 0; i < numMessages; i++) {
					TextMessage message = session0
							.createTextMessage("This is text message "
									+ messageCount++ + " " + (new Date()));

					producer.send(message);

					log.info("Sent message: " + message.getText()
							+ ", groupID: "
							+ message.getStringProperty("JMSXGroupID"));
					Thread.sleep(timeoutMillis);
				}

			} catch (NamingException e) {
				e.printStackTrace();
			} catch (JMSException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				log.info("Runner interrupted.");
				e.printStackTrace();
			} finally {
				try {
					connection.close();
					ic.close();
				} catch (JMSException e) {
					e.printStackTrace();
				} catch (NamingException e) {
					e.printStackTrace();
				}
			}

		}

		public int getMessageCount() {
			return messageCount;
		}
	}
}
